using System;
using System.Collections;

class BitSet32 : IEnumerable
{
	private uint mBits = 0;
	
	public void Set(int aIndex, bool aValue)
	{
		if (aIndex<0 || aIndex>31)
		{
			throw new ArgumentException("Invalid index!");
		}

		uint bitMask = (uint) 1 << aIndex;

		// Set bit aIndex to 0
		mBits = mBits & (~bitMask);

		if (aValue)
		{
			// Set bit aIndex to 1
			mBits = mBits | bitMask;
		}
	}

	public bool Get(int aIndex)
	{
		if (aIndex<0 || aIndex>31)
		{
			throw new ArgumentException("Invalid index!");
		}

		uint bitMask = (uint) 1 << aIndex;
		bool value = ((mBits & bitMask) != 0);
		return value;
	}

	public IEnumerator GetEnumerator()
	{
		return new BitSet32Enumerator(this);
	}

	class BitSet32Enumerator : IEnumerator
	{
		private BitSet32 mBitSet32;
		private int mCurrentIndex = -1;

		public BitSet32Enumerator(BitSet32 aBitSet32)
		{
			mBitSet32 = aBitSet32;
		}

		public bool MoveNext()
		{
			mCurrentIndex++;
			bool valid = (mCurrentIndex < 32);
			return valid;
		}

		public void Reset()
		{
			mCurrentIndex = -1;
		}

		public object Current
		{
			get
			{
				return mBitSet32.Get(mCurrentIndex);
			}
		}
	}
}

class TestBitSet32
{
	static void Main()
	{
		BitSet32 set = new BitSet32();
		set.Set(0, true);
		set.Set(31, true);
		set.Set(5, true);
		set.Set(5, false);
		set.Set(10, true);

		int index = 0;
		foreach (bool value in set)
		{
			Console.WriteLine("set[{0}] = {1}", index, value);
			index++;
		}
	}
}
